unit RegDBScr;

//---------------------------------------------------------------------------//
//                                                                           //
//   Registration unit for component DBScroll                                //
//                                                                           //
//   Date:    Oct, 10 1997                                                   //
//   Version: 1.1a                                                           //
//   Author:  Jan - Michael Strube                                           //
//   System:  Delphi 2.01 / C++Builder 1.0 / Delphi 3                        //
//                                                                           //
//   http://ourworld.compuserve.com/homepages/praxisservice/                 //
//                                                                           //
//---------------------------------------------------------------------------//

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DsgnIntf, DBScroll, ComCtrls, ExtCtrls, Menus;

type
  TDBScrollConnectionsDlg = class(TForm)
    BtnReConnect: TButton;
    BtnCancel: TButton;
    BtnOK: TButton;
    LvColumns: TListView;
    BtnDisconnect: TButton;
    Bevel1: TBevel;
    BtnDown: TButton;
    BtnUp: TButton;
    BtnDelete: TButton;
    PopupMenuDB_Fields: TPopupMenu;
    BtnAddModify: TButton;
    procedure BtnReConnectClick(Sender: TObject);
    procedure BtnDisconnectClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnUpClick(Sender: TObject);
    procedure BtnDownClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure LvColumnsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure BtnAddModifyClick(Sender: TObject);
  private
    ResetColumnAttributes: Boolean;
    ResetColumnWidth: Boolean;
  public
    Grid: TCustomDBScroll;
    DB_Fields: TStringList;
    constructor CreateIt(AOwner: TComponent; AGrid: TCustomDBScroll);
    destructor Destroy; override;
    procedure PopupMenuDB_FieldsClick(Sender: TObject);
  end;

  TDBScrollColumnProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses
  TypInfo, CommCtrl, DB;

{$R *.DFM}

//---------------------------------------------------------------------------//
//                              DataField Property                           //
//---------------------------------------------------------------------------//
type
  TDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result:= [paValueList, paSortList, paMultiSelect];
end;

procedure TDBStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values:= TStringList.Create;
  try
    GetValueList(Values);
    for I:= 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

type
  TDataFieldProperty = class(TDBStringProperty)
  public
    function GetDataSourcePropName: string; virtual;
    procedure GetValueList(List: TStrings); override;
  end;

function TDataFieldProperty.GetDataSourcePropName: string;
begin
  Result:= 'DataSource';
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TCustomDBScroll;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  // DataSource is a property of TDBScroll
  Instance:= TScrollColumn(GetComponent(0)).DBScrollGrid;
  if Instance <> nil then begin
    PropInfo:= TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then begin
      DataSource:= TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
      if (DataSource <> nil) and (DataSource.DataSet <> nil) then
        DataSource.DataSet.GetFieldNames(List);
    end;
  end;
end;

//---------------------------------------------------------------------------//
//                           ScrollColumn Property                           //
//---------------------------------------------------------------------------//
constructor TDBScrollConnectionsDlg.CreateIt(AOwner: TComponent; AGrid: TCustomDBScroll);
const
  DBSCR_LVS_EX_FULLROWSELECT = $00000020;
  DBSCR_LVM_GETEXTENDEDLISTVIEWSTYLE = LVM_FIRST + 55;
  DBSCR_LVM_SETEXTENDEDLISTVIEWSTYLE = LVM_FIRST + 54;
var
  LokZaehl: Integer;
  NewMenuItem: TMenuItem;
  Styles: DWORD;
begin
  Grid:= AGrid;
  DB_Fields:= TStringList.Create;
  if (Grid.DataSource <> nil) and (Grid.DataSource.DataSet <> nil) then
    Grid.DataSource.DataSet.GetFieldNames(DB_Fields);
  inherited Create(AOwner);
  for LokZaehl:= 0 to DB_Fields.Count-1 do begin
    NewMenuItem:= TMenuItem.Create(Self);
    NewMenuItem.Caption:= DB_Fields[LokZaehl];
    NewMenuItem.OnClick:= PopupMenuDB_FieldsClick;
    PopupMenuDB_Fields.Items.Add(NewMenuItem);
  end;
  ResetColumnAttributes:= False;
  ResetColumnWidth:= False;
  LvColumns.HandleNeeded;
  Styles:= LvColumns.Perform(DBSCR_LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0);
  LvColumns.Perform(DBSCR_LVM_SETEXTENDEDLISTVIEWSTYLE, 0, Styles or
                    DBSCR_LVS_EX_FULLROWSELECT);
end;

destructor TDBScrollConnectionsDlg.Destroy;
begin
  DB_Fields.Free;
  inherited;
end;

procedure TDBScrollConnectionsDlg.FormShow(Sender: TObject);
begin
  LvColumns.Selected:= LvColumns.Items[0];
  LvColumns.ItemFocused:= LvColumns.Selected;
end;

procedure TDBScrollConnectionsDlg.PopupMenuDB_FieldsClick(Sender: TObject);
begin
  if Sender is TMenuItem then begin
    if LvColumns.Selected.SubItems[0] = '' then
      // add column
      ResetColumnWidth:= True;
    LvColumns.Selected.SubItems[0]:= TMenuItem(Sender).Caption;
    LvColumns.Selected.SubItems[1]:= TMenuItem(Sender).Caption;
    LvColumns.Invalidate; 
  end;
end;

procedure TDBScrollConnectionsDlg.BtnReConnectClick(Sender: TObject);
var
  MaxCols: Integer;
  LokZaehl: Integer;
begin
  ResetColumnAttributes:= True;
  ResetColumnWidth:= True;
  BtnDisconnect.Click;
  with Grid do begin
    if DataSource = nil then begin
      MessageDlg('Property "' + Grid.Name + '.DataSource" has no valid value. Please '+
                 'check this property in the Object Inspector.', mtWarning, [mbOK], 0);
      ActiveControl:= BtnOK;
      Exit;
    end;
    if DataSource.DataSet = nil then begin
      MessageDlg('Property "'+ Grid.DataSource.Name + '.DataSet" of the connetced '+
                 'DataSource component has no valid value. Please check this '+
                 'property in the Object Inspector.', mtWarning, [mbOK], 0);
      ActiveControl:= BtnOK;
      Exit;
    end;
    if DB_Fields.Count < LvColumns.Items.Count then
      MaxCols:= DB_Fields.Count
    else
      MaxCols:= LvColumns.Items.Count;

    for LokZaehl:= 0 to MaxCols-1 do begin
      LvColumns.Items[LokZaehl].SubItems[0]:= DB_Fields[LokZaehl];
      LvColumns.Items[LokZaehl].SubItems[1]:= DB_Fields[LokZaehl];
    end;
  end;
  LvColumns.Invalidate;
end;

procedure TDBScrollConnectionsDlg.BtnDisconnectClick(Sender: TObject);
var
  LokZaehl: Integer;
begin
  with Grid do begin
    for LokZaehl:= 0 to LvColumns.Items.Count-1 do begin
      LvColumns.Items[LokZaehl].Caption:= Format('%.2d', [LokZaehl + 1]);
      LvColumns.Items[LokZaehl].SubItems[0]:= '';
      LvColumns.Items[LokZaehl].SubItems[1]:= '';
    end;
  end;
  LvColumns.Invalidate;
end;

procedure TDBScrollConnectionsDlg.BtnUpClick(Sender: TObject);
var
  ItemTxt: Array[1..3] of String;
  Idx: Integer;
begin
  if LvColumns.Selected <> nil then begin
    Idx:= LvColumns.Selected.Index;
    if Idx > 0 then begin
      ItemTxt[1]:= LvColumns.Items[Idx-1].Caption;
      ItemTxt[2]:= LvColumns.Items[Idx-1].SubItems[0];
      ItemTxt[3]:= LvColumns.Items[Idx-1].SubItems[1];
      LvColumns.Items[Idx-1].Caption:= LvColumns.Items[Idx].Caption;
      LvColumns.Items[Idx-1].SubItems[0]:= LvColumns.Items[Idx].SubItems[0];
      LvColumns.Items[Idx-1].SubItems[1]:= LvColumns.Items[Idx].SubItems[0];
      LvColumns.Items[Idx].Caption:= ItemTxt[1];
      LvColumns.Items[Idx].SubItems[0]:= ItemTxt[2];
      LvColumns.Items[Idx].SubItems[1]:= ItemTxt[3];
      LvColumns.Selected:= LvColumns.Items[Idx-1];
      LvColumns.ItemFocused:= LvColumns.Selected;
    end;
  end;
end;

procedure TDBScrollConnectionsDlg.BtnDownClick(Sender: TObject);
var
  ItemTxt: Array[1..3] of String;
  Idx: Integer;
begin
  if LvColumns.Selected <> nil then begin
    Idx:= LvColumns.Selected.Index;
    if Idx < LvColumns.Items.Count-1 then begin
      ItemTxt[1]:= LvColumns.Items[Idx+1].Caption;
      ItemTxt[2]:= LvColumns.Items[Idx+1].SubItems[0];
      ItemTxt[3]:= LvColumns.Items[Idx+1].SubItems[1];
      LvColumns.Items[Idx+1].Caption:= LvColumns.Items[Idx].Caption;
      LvColumns.Items[Idx+1].SubItems[0]:= LvColumns.Items[Idx].SubItems[0];
      LvColumns.Items[Idx+1].SubItems[1]:= LvColumns.Items[Idx].SubItems[0];
      LvColumns.Items[Idx].Caption:= ItemTxt[1];
      LvColumns.Items[Idx].SubItems[0]:= ItemTxt[2];
      LvColumns.Items[Idx].SubItems[1]:= ItemTxt[3];
      LvColumns.Selected:= LvColumns.Items[Idx+1];
      LvColumns.ItemFocused:= LvColumns.Selected;
    end;
  end;
end;

procedure TDBScrollConnectionsDlg.BtnAddModifyClick(Sender: TObject);
var
  ListEntryRect: TRect;
  CurrentPos: TPoint;
begin
  if LvColumns.Selected <> nil then begin
    ListEntryRect:= LvColumns.Selected.DisplayRect(drBounds);
    CurrentPos:= Point(ListEntryRect.Left + 100, ListEntryRect.Bottom);
    CurrentPos:= LvColumns.ClientToScreen(CurrentPos);
    PopupMenuDB_Fields.Popup(CurrentPos.X, CurrentPos.Y);
  end;
end;

procedure TDBScrollConnectionsDlg.BtnDeleteClick(Sender: TObject);
var
  Idx: Integer;
begin
  if LvColumns.Selected <> nil then begin
    Idx:= LvColumns.Selected.Index;
    LvColumns.Items[Idx].SubItems[0]:= '';
    LvColumns.Items[Idx].SubItems[1]:= '';
    ResetColumnWidth:= True;
    if LvColumns.Items.Count > LvColumns.Selected.Index+1 then begin
      LvColumns.Selected:= LvColumns.Items[LvColumns.Selected.Index+1];
      LvColumns.ItemFocused:= LvColumns.Selected;
    end;
  end;
  LvColumns.Invalidate;
end;

procedure TDBScrollConnectionsDlg.LvColumnsKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_DELETE then BtnDelete.Click;
end;

procedure TDBScrollConnectionsDlg.BtnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TDBScrollConnectionsDlg.BtnOKClick(Sender: TObject);
var
  LokZaehl: Integer;
  OldPos: Integer;
  TempColumns: Array[1..DBScrMaxColumns] of TScrollColumn;
  DsgnForm: {$IFDEF VER100} TCustomForm {$ELSE} TForm {$ENDIF};
  ConnectedCols: Integer;
  TempField: TField;
begin
  ConnectedCols:= 0;
  with Grid do begin
    for LokZaehl:= 1 to LvColumns.Items.Count do begin
      OldPos:= StrToInt(LvColumns.Items[LokZaehl-1].Caption);
      TempColumns[LokZaehl]:= ColumnXX[OldPos];
      TempColumns[LokZaehl].DataField:= LvColumns.Items[LokZaehl-1].SubItems[0];
      TempColumns[LokZaehl].Header:= LvColumns.Items[LokZaehl-1].SubItems[1];
      if TempColumns[LokZaehl].DataField <> '' then inc(ConnectedCols);
      if ResetColumnAttributes then begin
        if DataLink.DataSet <> nil then begin
          TempField:= DataLink.DataSet.FindField(TempColumns[LokZaehl].DataField);
          if TempField <> nil then begin
            TempColumns[LokZaehl].Alignment:= TempField.Alignment;
            TempColumns[LokZaehl].ReadOnly:= TempField.ReadOnly
          end else begin
            TempColumns[LokZaehl].Alignment:= taLeftJustify;
            TempColumns[LokZaehl].ReadOnly:= False;
          end;
        end else begin
          TempColumns[LokZaehl].Alignment:= taLeftJustify;
          TempColumns[LokZaehl].ReadOnly:= False;
        end;
        TempColumns[LokZaehl].CanEnter:= True;
        TempColumns[LokZaehl].DefaultDrawing:= True;
        TempColumns[LokZaehl].DisplayLength:= 1000;
        TempColumns[LokZaehl].HeaderAlignment:= taCenter;
        TempColumns[LokZaehl].OwnerDraw:= False;
        TempColumns[LokZaehl].ParentColor:= True;
      end;
    end;

    for LokZaehl:= 1 to DBScrMaxColumns do begin
      ColumnXX[LokZaehl]:= TempColumns[LokZaehl];
      if ResetColumnWidth then begin
        if ColumnXX[LokZaehl].DataField = '' then
          ColumnXX[LokZaehl].SetWidthPercent2(0)
        else
          if ConnectedCols > 0 then
            ColumnXX[LokZaehl].SetWidthPercent2(100 / ConnectedCols);
      end;
    end;
    CalcRectArea;
    if Grid.HandleAllocated then InvalidateRect(Grid.Handle, nil, False);
  end;
  DsgnForm:= GetParentForm(Grid);
  if DsgnForm <> nil then DsgnForm.Designer.Modified;
  Close;
end;

procedure TDBScrollColumnProperty.Edit;
var
  LokZaehl: Integer;
begin
  with TDBScrollConnectionsDlg.CreateIt(Application,
             TScrollColumn(GetOrdValue).Grid) do try
    for LokZaehl:= 1 to DBScrMaxColumns do begin
      with LvColumns.Items.Add do begin
        Caption:= Format('%.2d', [LokZaehl]);
        SubItems.Add(Grid.ColumnXX[LokZaehl].DataField);
        SubItems.Add(Grid.ColumnXX[LokZaehl].Header);
      end;
    end;
    ShowModal;
  finally
    Free;
  end;
end;

function TDBScrollColumnProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paSubProperties];
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(String), TScrollColumn, 'DataField', TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(TScrollColumn), nil, '', TDBScrollColumnProperty);
  RegisterComponents('PrxService', [TDBScroll]);
end;

end.
